local MODNAME = 'TrackingAuras'
local ShivvAudio = LibStub('AceAddon-3.0'):GetAddon('ShivvAudio')
local TrackingAuras = ShivvAudio:NewModule(MODNAME, 'AceEvent-3.0')

local db
local activeAuras, partyGUID = {}, {['party1'] = 0, ['party2'] = 0, ['party3'] = 0, ['party4'] = 0}

function TrackingAuras:OnInitialize()
	self:SetEnabledState(ShivvAudio.db.profile.trackingAuras)
end

function TrackingAuras:OnEnable()
	db = ShivvAudio.db.profile.auras -- local lookup ref

	-- clear out old active tracking entries
	for unit in pairs(activeAuras) do
		for auras in pairs(activeAuras[unit]) do
			activeAuras[unit][auras] = nil
		end
		activeAuras[unit] = nil
	end

	-- setup active tracking entries
	for unit in pairs(db) do
		activeAuras[unit] = {}

		-- silently scan tracked units to populate data
		if (UnitExists(unit)) then
			self:ScanAuras(unit, true)
		end

		if (strsub(unit, 1, 5) == 'party') then -- store party guid for future changes
			partyGUID[unit] = UnitGUID(unit)
		end
	end

	self:UNIT_PET(nil, 'player') -- check for pet

	self:RegisterEvent('UNIT_AURA')
	self:RegisterEvent('UNIT_PET')
	self:RegisterEvent('GROUP_ROSTER_UPDATE')
	self:RegisterEvent('PLAYER_TARGET_CHANGED')
	self:RegisterEvent('PLAYER_FOCUS_CHANGED')
end

function TrackingAuras:OnDisable()
	self:UnregisterEvent('UNIT_AURA')
	self:UnregisterEvent('UNIT_PET')
	self:UnregisterEvent('PLAYER_FOCUS_CHANGED')
	self:UnregisterEvent('PLAYER_TARGET_CHANGED')
	self:UnregisterEvent('GROUP_ROSTER_UPDATE')
end

-- ------------ --
-- AURA SCANNER --
-- ------------ --
do
	local GetTime, UnitBuff, UnitDebuff = _G.GetTime, _G.UnitBuff, _G.UnitDebuff -- local references for speed
	local aura, i, time, unitData, activeData

	function TrackingAuras:ScanAuras(unit, silent)
		unitData, activeData = db[unit], activeAuras[unit] -- local references for speed
		time = GetTime()

		-- scan buffs on unit for tracked auras
		i, aura = 1, UnitBuff(unit, 1)

		while(aura) do
			if (unitData[aura]) then -- found a buff we are tracking
				if (activeData[aura]) then -- already being tracked, update timestamp
					activeData[aura] = time
				else -- not being tracked, its new!
					activeData[aura] = time

					if (not silent and unitData[aura].onGain) then -- this isn't a silent scan and an onGain sound exists
						ShivvAudio:SoundHandler(unitData[aura].onGain)
					end

					if (unit == 'player') then -- scanning player
						ShivvAudio:MusicStart(aura)
					end
				end
			end

			i = i + 1
			aura = UnitBuff(unit, i)
		end

		-- now do the exact same thing but for debuffs this time, ugh
		i, aura = 1, UnitDebuff(unit, 1)

		while(aura) do
			if (unitData[aura]) then -- found a buff we are tracking
				if (activeData[aura]) then -- already being tracked, update timestamp
					activeData[aura] = time
				else -- not being tracked, its new!
					activeData[aura] = time

					if (not silent and unitData[aura].onGain) then -- this isn't a silent scan and an onGain sound exists
						ShivvAudio:SoundHandler(unitData[aura].onGain)
					end

					if (unit == 'player') then -- scanning player
						ShivvAudio:MusicStart(aura)
					end
				end
			end

			i = i + 1
			aura = UnitDebuff(unit, i)
		end

		-- we've found all new buffs, lets get rid of any old ones
		for aura, timestamp in pairs(activeData) do
			if (timestamp ~= time) then -- aura is no longer active, remove it
				activeData[aura] = nil

				if (not silent and unitData[aura].onLoss) then -- not silent and an onLoss sound exists
					ShivvAudio:SoundHandler(unitData[aura].onLoss)
				end

				if (unit == 'player') then -- scanning player
					ShivvAudio:MusicStop(aura)
				end
			end
		end
	end
end

function TrackingAuras:UNIT_AURA(event, unit)
	if (activeAuras[unit]) then
		self:ScanAuras(unit)
	end
end

function TrackingAuras:PLAYER_FOCUS_CHANGED()
	if (activeAuras['focus'] and UnitExists('focus')) then -- do something if we're tracking the focus and it exists
		self:ScanAuras('focus', true) -- scan silently
	end
end

function TrackingAuras:PLAYER_TARGET_CHANGED()
	if (activeAuras['target'] and UnitExists('target')) then -- do something if we're tracking the target and it exists
		self:ScanAuras('target', true) -- scan silently
	end
end

function TrackingAuras:GROUP_ROSTER_UPDATE()
	for unit in pairs(partyGUID) do
		if (activeAuras[unit] and UnitExists(unit)) then -- if we're tracking this party member and they exist
			if (partyGUID ~= UnitGUID(unit)) then -- either a different party member or is one where there wasn't before
				self:ScanAuras(unit, true) -- scan silently
				partyGUID[unit] = UnitGUID(unit)
			end
		else -- no party member here
			partyGUID[unit] = 0
		end
	end
end

do
	local petGUID = 0

	function TrackingAuras:UNIT_PET(event, unit)
		if (unit ~= 'player') then return end

		if (activeAuras['pet'] and UnitExists('pet')) then
			if (petGUID ~= UnitGUID('pet')) then -- either a different pet or have one when didn't before
				self:ScanAuras('pet', true) -- scan silently
				petGUID = UnitGUID('pet')
			end
		else -- pet doesn't exist, clear guid
			petGUID = 0
		end
	end
end
